/*
 ******************************************************************************
 *
 * This file is a part of the IBIS ICM Parser.
 * Copyright (C) 2003 Kelly C. Green <kgreen22@mindspring.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 * 
 ******************************************************************************
 *
 * $Header: /home/kgreen/cvsroot/icmchk/src/ibis-icm.c,v 1.1.1.1 2003/11/12 05:00:51 kgreen Exp $
 *
 ******************************************************************************
 *
 * $Log: ibis-icm.c,v $
 * Revision 1.1.1.1  2003/11/12 05:00:51  kgreen
 * Importing Project
 *
 *
 ******************************************************************************
 */

#if ! (defined(__GNUC__) || defined(__attribute__))
#define __attribute__(a) /* a */
#endif

static char *CVSID __attribute__((unused)) = "$Id: ibis-icm.c,v 1.1.1.1 2003/11/12 05:00:51 kgreen Exp $";

/*
 * Driver for the IBIS ICM parser
 */

char *copyright = \
"\
\n\
  This program is the IBIS ICM Parser version %s\n\
  Copyright (C) 2003 Kelly C. Green <kgreen22@mindspring.com>\n\
\n\
  This program is free software; you can redistribute it and/or modify\n\
  it under the terms of the GNU General Public License as published by\n\
  the Free Software Foundation; either version 2 of the License, or\n\
  (at your option) any later version.\n\
\n\
  You may also contact the author to secure a non-public license.\n\
  A fee will be required for this service.\n\
\n\
  This program is distributed in the hope that it will be useful, but\n\
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY\n\
  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License\n\
  for more details.\n\
\n\
  You should have received a copy of the GNU General Public License along\n\
  with this program; if not, write to the Free Software Foundation, Inc.,\n\
  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n\
\n\
";

#include "ibis-icm.h"
#include <stdio.h>

extern FILE *yyin;

static char *ICM_filename = (char *)0;
static char *ICM_pathname = "";
static char *ICM_progname = (char *)0;

static void usage (void);

int main (int argc, char **argv)
{
  char *sep;
  int path_len;
  int sort = 0;
  int pipe = 0;
  
  yyin = stdin;               /* assume a pipeline for now */
  set_verbosity(1);           /* set minimum verbosity (warnings) */

  sep = strrchr(argv[0], SEPARATOR[0]);
  if (!sep) {
    ICM_progname = argv[0];
  } else {
    sep[0] = '\0';
    ICM_progname = &sep[1];
  }

  while ( --argc > 0 ) {
    argv++;
    switch ((*argv)[0]) {
    case '-':  
      switch ((*argv)[1]) {
      case 'v': 
	set_verbosity(get_verbosity() + 1);
	break;
      case 'c': 
	fprintf(stdout, copyright, IBIS_ICM_VERSION);
	exit(0);
      case 's': 
	sort = 1;
	break;
      case 'p':
	pipe = 1;
	break;
      case 'h':
	usage();
	exit(0);
	break;
      default:
	usage();
	exit(1);
      }
      break;
    default:

      /*
       * can't have both a pipe and a filename
       */

      if (ICM_filename || pipe) {
	usage();
	exit(1);
      }

      yyin = freopen( argv[0], "r" , stdin);
      if (!yyin) {
	perror(argv[0]);
	exit(1);
      }

      /*
       * Strip the file name from the path name.
       *
       * We need the path to support a check for file existance 
       * in any file references within the given ICM file.
       *
       * The path separator is OS dependent.
       */
      
      sep = strrchr(argv[0], SEPARATOR[0]);
      if (!sep) {
	ICM_filename = argv[0];
	ICM_pathname = "";
      } else {
	sep[0] = '\0';
	path_len = strlen(argv[0]);
	ICM_filename = &sep[1];
	ICM_pathname = calloc(1, path_len + 1 + 1);
	strncat(ICM_pathname, argv[0], path_len);
	ICM_pathname[path_len] = SEPARATOR[0];
	ICM_pathname[path_len + 1] = '\0';
      }
    } 
  }

  /*
   * must have either a pipe or a filename
   */

  if (!(pipe || ICM_filename)) {
    usage();
    exit(1);
  }

  message_init(sort);         /* initialize message subsystem */
  yylex_init();               /* initialize the lexical analyzer */
  fprintf(stderr, "IBIS ICM Parser, version %s\n", IBIS_ICM_VERSION);
  
  yyparse();                  /* parse the input file */

  message_finish();           /* in-order print out of all messages */
  fprintf(stderr, "Finished. ERRORS: %d", get_error_count());
  if (get_verbosity() >=1) fprintf(stderr, " WARNINGS: %d", get_warning_count());
  if (get_verbosity() >=2) fprintf(stderr, " INFORMATIONAL: %d", get_info_count());
  if (get_verbosity() >=3) fprintf(stderr, " DEBUG: %d", get_debug_count());
  fprintf(stderr, "\n");
  
  exit (0 != get_error_count());
}

/*
 * a semi-friendly usage message
 */

static void usage (void)
{
fprintf(stdout,"\n\
IBIS ICM Version %s parser.\n\n\
USAGE: %s -v -s -c -h -p <file name>\n\
       -v --> (verbose, once)   Report additional informational messages.\n\
       -v --> (verbose, twice)  Report debug messages (caution: voluminous).\n\
       -s --> (sort)            Sort diagnostic messages in order of line number. \n\
       -c --> (copyright)       Display program copyright and exit.\n\
       -h --> (help)            Display this message and exit.\n\
       -p --> (pipe)            Read stdin pipeline instead of file.\n\
       <file name>              The input IBIS ICM Model file.\n\n",
  IBIS_ICM_VERSION, ICM_progname);
}


/*
 * determine if an ICM specified  file exists.
 * automagically looks in the directory same directory as the .icm file
 *
 * returns  1 if the file does not exist in the same directory as the .icm file
 * returns  0 if the file does exist in the same directory as the .icm file
 * returns -1 if the file does not exist in the CWD, and we're reading from a pipeline.
 */

int file_exists (char *name)
{
  
  char *filename;
  int path_len;
  int name_len;
  char *pos;
  FILE *fp;
  
  if (!(name && strlen(name))) return 0;  /* no file name?  no problem! */
  
  path_len = strlen(ICM_pathname);
  name_len = strlen(name);
  
  filename = calloc(1, path_len + 1 + name_len + 1);
  pos = filename;
  
  if (path_len) {
    strncat(pos, ICM_pathname, path_len);
    pos += path_len;
    strncat(pos, SEPARATOR, 1);
    pos += 1;
  }
  strncat(pos, name, name_len); 
  
  fp = fopen(filename, "r");
  free(filename);
  if (fp) {
    fclose(fp);
    return 0;
  }
  
  if (ICM_filename) {
    return 1;
  } else {
    return -1;
  }
}

char *icm_file_name (void)
{
  return ICM_filename;
}

char *icm_dir_name (void)
{
  return ICM_pathname;
}
 

