/*
 ******************************************************************************
 *
 * This file is a part of the IBIS ICM Parser.
 * Copyright (C) 2003 Kelly C. Green <kgreen22@mindspring.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 * 
 ******************************************************************************
 *
 * $Header: /home/kgreen/cvsroot/icmchk/src/messages.c,v 1.1.1.1 2003/11/12 05:00:51 kgreen Exp $
 *
 ******************************************************************************
 *
 * $Log: messages.c,v $
 * Revision 1.1.1.1  2003/11/12 05:00:51  kgreen
 * Importing Project
 *
 *
 ******************************************************************************
 */

#if ! (defined(__GNUC__) || defined(__attribute__))
#define __attribute__(a) /* a */
#endif

static char *CVSID __attribute__((unused)) = "$Id: messages.c,v 1.1.1.1 2003/11/12 05:00:51 kgreen Exp $";

#include "ibis-icm.h"
#include "btree.h"
#include "dynstr.h"
#include <stdarg.h>

enum level_e {
  UNUSED = 0, INFO = 1, WARNING = 2, ERROR = 3, DEBUG = 4
};

typedef enum level_e level_t;

static char *level_names[] = { "UNUSED", "INFO", "WARNING", "ERROR", "DEBUG" }; 

static void vmsg  PROTO((YYLTYPE *loc, level_t l, char *s, va_list ap));

static int num_errors;
static int num_warnings;
static int num_infos;
static int num_debugs;

static int verbosity;

/*
 * Error messages are stored in a binary tree, ordered by line number.
 * They are all printed out in order after the parse is complete
 *
 * If this isn't done, messages would appear at time of discovery,
 * in order of random line numbers, which is confusing.
 *
 * What follows is the binary tree root, and some small routines
 * to manage it's elements, which the btree internally structures
 * as a collection of opaque objects of a uniform type 
 * referenced by void *.
 */

typedef struct message_s message_t;

struct message_s {
  YYLTYPE loc;
  int count;
  level_t level;
  char *text;
};

static root_t message_table = (root_t) 0;

static int message_compare (void *item1, void *item2)
{
  message_t *message_1 = (message_t *) item1;
  message_t *message_2 = (message_t *) item2;

  /*
   * sort by line number first
   */
  
  if (message_1->loc.first_line != message_2->loc.first_line) {
    return(message_2->loc.first_line - message_1->loc.first_line);
  }

  /*
   * same line number - sort by column number
   */
  
  if (message_1->loc.first_column != message_2->loc.first_column) {
    return(message_2->loc.first_column - message_1->loc.first_column);
  }
  
  /*
   * same line/col number, sort by message level
   */
  
  if (message_1->level != message_2->level) {
    return(message_2->level - message_1->level);
  }
  
  /*
   * same line/column number and severity - sort by message text
   */
  
  return (dyn_strcasecmp(message_2->text, message_1->text));
}

static void *message_create (void *item)
{
  message_t *message = (message_t *)item;
  message_t *new_message = (message_t *) calloc(1, sizeof(message_t));
  
  new_message->level = message->level;
  new_message->loc = message->loc;
  new_message->text = strdup(message->text);
  return (void *) new_message;
}

static void message_delete (void *item)
{
  message_t *message = (message_t *)item;
  free(message->text);
  free(message);
}

/*
 * Parser error reporting functions
 */

void set_verbosity (int v)
{
  verbosity = v;
}

int get_verbosity (void)
{
  return verbosity;
}

int get_error_count (void)
{
  return num_errors;
}

int get_warning_count (void)
{
  return num_warnings;
}

int get_info_count (void)
{
  return num_infos;
}

int get_debug_count (void)
{
  return num_debugs;
}

void yyerror (YYLTYPE *loc, char *s)
{
  err(loc, s);
}


void err (YYLTYPE *loc, char *s, ...)
{
  va_list ap;
  va_start(ap, s);
  vmsg(loc, ERROR, s, ap);
  va_end(ap);
  num_errors++;
}


void warn (YYLTYPE *loc, char *s, ...)
{
  if (verbosity >= 1) {
    va_list ap;
    va_start(ap, s);
    vmsg(loc, WARNING, s, ap);
    va_end(ap);
  }
  num_warnings++;
}


void info (YYLTYPE *loc, char *s, ...)
{
  if (verbosity >= 2) {
    va_list ap;
    va_start(ap, s);
    vmsg(loc, INFO, s, ap);
    va_end(ap);
  }
  num_infos++;
}


void debug (YYLTYPE *loc, char *s, ...)
{
  if (verbosity >= 3) {
    va_list ap;
    va_start(ap, s);
    vmsg(loc, DEBUG, s, ap);
    va_end(ap);
  }
  num_debugs++;
}

void message_init (int sort)
{
  if (message_table) btree_free(&message_table);
  if (sort) message_table = btree_alloc(message_compare, message_create, message_delete);
  num_errors = 0;
  num_warnings = 0;
  num_infos = 0;
  num_debugs = 0;
}

static void vmsg (YYLTYPE *loc, level_t l, char *s, va_list ap)
{
  message_t *msg = (message_t *) calloc(1, sizeof(message_t));

  /*
   * Note - I would use snprintf and vsnprintf, but they are
   * not available under the C89 standard.
   *
   * There is some risk of overflowing the following message
   * buffer, but it is small.  I control the formats, and this
   * buffer is way larger than I ever need.  The user can only 
   * directly control the length of object names going into
   * the variable argument list, and would have a pretty tough 
   * time making names long enough to cause problems within
   * a 120 character line limit.
   */

  char text[1024];

  if (loc) {
    msg->loc = *loc;
  }
  msg->level = l;
  vsprintf(text, s, ap);
  msg->text = strdup(text);

  if (message_table) {
    msg = btree_search(message_table, (void *) msg);
    msg->count += 1;  /* count references */
  } else {
    fprintf(stderr, "%s (Line %i/Col %i) %s\n", level_names[msg->level], msg->loc.first_line, msg->loc.first_column, msg->text);
    free(msg->text);
    free(msg);
  }
}

static void message_print (void *message, void *data)
{
  message_t *msg = (message_t *)message;
  int i;
  for (i = msg->count; i > 0; i--) {
    fprintf(stderr, "%s (Line %i/Col %i) %s\n", level_names[msg->level], msg->loc.first_line, msg->loc.first_column, msg->text);
  }
}

void message_finish (void)
{
  btree_walk(message_table, message_print, (void *)0);
}

